//
// To compile in Visual Studio 2022 (free Community Edition): 
//   1) Create a new Dynamic Link Library project
//   2) Copy this .c file into the Project Directory that was created in #1 above
//   3) Remove all other Source Files added by #1 above (dllmain.cpp, psh.h, etc.)
//   4) Use Project -> Add Existing Item to select and add this .c file to the Source Files
//   5) Change Active Solution Platform from x64 to x86 in Configuration Manager
//   6) turn off the Use Precompiled Headers option in Project -> "ProjectName" Properties -> C/C++ -> Precompiled Heading section -> Precompiled Header -> Not Using Precompiled Headers
//   7) Choose Build Solution
//   8) .dll file compiled into \debug project directory will need to be copied to C:\NeuroShell Trader\Template directory
//      for the .dll file to be seen/used by a NeuroShell Trader chart.
//

#include <windows.h>   
#include <math.h>   

const float MISSING = (float) 3.4e38;    // NeuroShell Trader missing value constant

_declspec(dllexport) void ElegantOscillator(float* Close, long BandEdge, long numrows, float* output)
{	// The Elegant Oscillator
	// (c)2021 John F.Ehlers

	double* Deriv;
	double RMS=0.0, NDeriv=0.0, IFish=0.0, IFish1=0.0, a1=0.0, b1=0.0, c1=0.0, c2=0.0, c3=0.0, SS=0.0, SS1=0.0, SS2=0.0;
	long Currentbar, count;
	double radiansconvertmult = 3.1415926535 / 180.0;

	Deriv = malloc(sizeof(double) * numrows);

	for (Currentbar = 0; Currentbar < numrows; Currentbar++)  
	{
		if (Currentbar >= 2)
		{	//Take the derivative of prices
			Deriv[Currentbar] = Close[Currentbar] - Close[Currentbar - 2];

			if (Currentbar > 50)
			{	//Normalize to standard deviation
				RMS = 0;
				for (count = 0; count <= 49; count++)
					RMS = RMS + Deriv[Currentbar - count] * Deriv[Currentbar - count];

				if (RMS != 0)
					RMS = sqrt(RMS / 50);
				NDeriv = Deriv[Currentbar] / RMS;

				//Compute the Inverse Fisher Transform
				IFish = (exp(2 * NDeriv) - 1) / (exp(2 * NDeriv) + 1);

				//Integrate with SuperSmoother
				a1 = exp(-1.414 * 3.14159 / BandEdge);
				b1 = 2 * a1 * cos(radiansconvertmult * 1.414 * 180 / BandEdge);
				c2 = b1;
				c3 = -a1 * a1;
				c1 = 1 - c2 - c3;
				SS = c1 * (IFish + IFish1) / 2 + c2 * SS1 + c3 * SS2;
				if (Currentbar < 2)
					SS = 0;

				output[Currentbar] = (float)SS;

				// save lag of variables
				SS2 = SS1;
				SS1 = SS;
				IFish1 = IFish;
			}
		}
	}

	free(Deriv);
}